@@ -25,6 +25,11 @@
 #include "document/document_glom.h"
 #include "data_structure/glomconversions.h"
 #include "mode_data/dialog_choose_field.h"
+//#include "dialog_layout_report.h"
+#include "utils.h"
+#include "data_structure/glomconversions.h"
+#include "data_structure/layout/report_parts/layoutitem_summary.h"
+#include "data_structure/layout/report_parts/layoutitem_fieldsummary.h"
 #include <glibmm/i18n.h>
 //#include <libgnomeui/gnome-app-helper.h>
 
@@ -131,7 +136,17 @@ Glib::RefPtr<Gnome::Gda::DataModel> Base_DB::Query_execute(const Glib::ustring&
   {
     Glib::RefPtr<Gnome::Gda::Connection> gda_connection = sharedconnection->get_gda_connection();
 
-    std::cout << "Query_execute():  " << strQuery << std::endl;
+    /*
+    try
+    {
+      std::cout << "Debug: Query_execute():  " << strQuery << std::endl;
+    }
+    catch(const Glib::Exception& ex)
+    {
+      std::cout << "Debug: query string could not be converted to std::cout: " << ex.what() << std::endl;
+    }
+    */
+    
     result = gda_connection->execute_single_command(strQuery);
     if(!result)
     {
@@ -150,7 +165,8 @@ void Base_DB::load_from_document()
 {
   if(get_document())
   {
-    fill_from_database(); //virtual.
+    if(ConnectionPool::get_instance()->get_ready_to_connect())
+      fill_from_database(); //virtual.
 
     //Call base class:
     View_Composite_Glom::load_from_document();
@@ -168,7 +184,7 @@ bool Base_DB::get_table_exists_in_database(const Glib::ustring& table_name) cons
   return result;
 }
 
-Base_DB::type_vecStrings Base_DB::get_table_names() const
+Base_DB::type_vecStrings Base_DB::get_table_names(bool ignore_system_tables) const
 {
   type_vecStrings result;
 
@@ -194,9 +210,23 @@ Base_DB::type_vecStrings Base_DB::get_table_names() const
         {
           table_name = value.get_string();
 
+          bool add_it = true;
+          
+          if(ignore_system_tables)
+          {
+            //Check whether it's a system table:
+            const Glib::ustring prefix = "glom_system_";
+            const Glib::ustring table_prefix = table_name.substr(0, prefix.size());
+            if(table_prefix == prefix)
+              add_it = false;
+          }
+          
           //Ignore the pga_* tables that pgadmin adds when you use it:
-          if(table_name.substr(0, 4) != "pga_")
-            result.push_back( table_name );
+          if(table_name.substr(0, 4) == "pga_")
+            add_it = false;
+            
+          if(add_it)
+            result.push_back(table_name);
         }
       }
     }
@@ -348,8 +378,13 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table_from_database(const Glib::
         //Get the field name:
         Gnome::Gda::Value value_name = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_NAME, row);
         if(value_name.get_value_type() ==  Gnome::Gda::VALUE_TYPE_STRING)
+        {
+          if(value_name.get_string().empty())
+            g_warning("Base_DB::get_fields_for_table_from_database(): value_name is empty.");
+            
           field_info.set_name( value_name.get_string() );
-
+        }
+        
         //Get the field type:
         //This is a string representation of the type, so we need to discover the Gnome::Gda::ValueType for it:
         Gnome::Gda::Value value_fieldtype = data_model_fields->get_value_at(DATAMODEL_FIELDS_COL_TYPE, row);
@@ -432,7 +467,7 @@ Base_DB::type_vecFields Base_DB::get_fields_for_table(const Glib::ustring& table
     //Look at each field in the database:
     for(type_vecFields::iterator iter = fieldsDocument.begin(); iter != fieldsDocument.end(); ++iter)
     {
-      Glib::ustring field_name = iter->get_name();
+      const Glib::ustring field_name = iter->get_name();
 
       //Get the field info from the database:
       //This is in the document as well, but it _might_ have changed.
@@ -740,6 +775,7 @@ void Base_DB::add_standard_tables() const
   {
     TableInfo prefs_table_info;
     prefs_table_info.m_name = GLOM_STANDARD_TABLE_PREFS_TABLE_NAME;
+    prefs_table_info.m_title = _("System: Preferences");
     prefs_table_info.m_hidden = true;
 
     Document_Glom::type_vecFields pref_fields;
@@ -803,6 +839,7 @@ void Base_DB::add_standard_tables() const
   {
     TableInfo table_info;
     table_info.m_name = GLOM_STANDARD_TABLE_AUTOINCREMENTS_TABLE_NAME;
+    table_info.m_title = _("System: Auto Increments");
     table_info.m_hidden = true;
 
     Document_Glom::type_vecFields fields;
@@ -1136,6 +1173,9 @@ void Base_DB::set_database_preferences(const SystemPrefs& prefs)
     {
       std::cerr << "Base_DB::set_database_preferences(): exception: " << ex.what() << std::endl;
     }
+    
+    //Set some information in the document too, so we can use it to recreate the database:
+    get_document()->set_database_title(prefs.m_name);
 }
 
 bool Base_DB::create_table(const TableInfo& table_info, const Document_Glom::type_vecFields& fields) const
@@ -1248,3 +1288,288 @@ Glib::ustring Base_DB::get_layout_item_table_name(const LayoutItem_Field& layout
 
   return Glib::ustring();
 }
+
+void Base_DB::report_build_summary(const Glib::ustring& table_name, xmlpp::Element& parent_node, LayoutItem_Summary& summary, const Glib::ustring& where_clause)
+{
+  //Add XML node:
+  xmlpp::Element* nodeSummary = parent_node.add_child("summary");
+
+  //Get fields
+  GlomUtils::type_vecLayoutFields fieldsToGet;
+  for(LayoutGroup::type_map_items::iterator iterChildren = summary.m_map_items.begin(); iterChildren != summary.m_map_items.end(); ++iterChildren)
+  {
+    LayoutItem_Field* pField = dynamic_cast<LayoutItem_Field*>(iterChildren->second);
+    if(pField)
+    {
+      fill_full_field_details(table_name, *pField);
+      fieldsToGet.push_back( sharedptr<LayoutItem_Field>( static_cast<LayoutItem_Field*>(pField->clone() ) ) );
+    }
+    else
+    {
+      LayoutItem_GroupBy* pGroupBy = dynamic_cast<LayoutItem_GroupBy*>(iterChildren->second);
+      if(pGroupBy)
+      {
+        //Recurse, adding a sub-groupby block:
+        report_build_groupby(table_name, *nodeSummary, *pGroupBy, where_clause);
+      }
+      else
+      {
+        LayoutItem_Summary* pSummary = dynamic_cast<LayoutItem_Summary*>(iterChildren->second);
+        if(pSummary)
+        {
+          //Recurse, adding a summary block:
+          report_build_summary(table_name, *nodeSummary, *pSummary, where_clause);
+        }
+      }
+    }
+  }
+
+  if(!fieldsToGet.empty())
+  {
+    //Rows, with data:
+    report_build_records(table_name, *nodeSummary, fieldsToGet, where_clause, Glib::ustring() /* No sort_clause because there is only one row */);
+  }
+}
+
+void Base_DB::report_build_groupby(const Glib::ustring& table_name, xmlpp::Element& parent_node, LayoutItem_GroupBy& group_by, const Glib::ustring& where_clause_parent)
+{
+  //Get the possible heading values.
+  LayoutItem_Field* field_group_by = group_by.get_field_group_by();
+  if(field_group_by)
+  {
+    fill_full_field_details(table_name, *field_group_by);
+
+    //Get the possible group values, ignoring repeats by using GROUP BY.
+    const Glib::ustring group_field_table_name = get_layout_item_table_name(*field_group_by, table_name);
+    Glib::ustring sql_query = "SELECT " + group_field_table_name + "." + field_group_by->get_name() +
+      " FROM " + group_field_table_name;
+
+    if(!where_clause_parent.empty())
+      sql_query += " WHERE " + where_clause_parent;
+
+    sql_query += " GROUP BY " + field_group_by->get_name(); //rTODO: And restrict to the current found set.
+
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(datamodel)
+    {
+      guint rows_count = datamodel->get_n_rows();
+      for(guint row = 0; row < rows_count; ++row)
+      {
+        const Gnome::Gda::Value group_value = datamodel->get_value_at(0 /* col*/, row);
+
+        //Add XML node:
+        xmlpp::Element* nodeGroupBy = parent_node.add_child("group_by");
+
+        nodeGroupBy->set_attribute("group_field", field_group_by->get_title_or_name());
+        nodeGroupBy->set_attribute("group_value",
+          GlomConversions::get_text_for_gda_value(field_group_by->m_field.get_glom_type(), group_value, field_group_by->get_formatting_used().m_numeric_format) );
+
+        Glib::ustring where_clause = group_field_table_name + "." + field_group_by->get_name() + " = " + field_group_by->m_field.sql(group_value);
+        if(!where_clause_parent.empty())
+          where_clause += " AND (" + where_clause_parent + ")";
+
+        //Secondary fields. For instance, the Contact Name, in addition to the Contact ID that we group by.
+        if(!(group_by.m_group_secondary_fields.m_map_items.empty()))
+        {
+          xmlpp::Element* nodeSecondaryFields = nodeGroupBy->add_child("secondary_fields");
+
+          GlomUtils::type_vecLayoutFields fieldsToGet;
+          for(LayoutGroup::type_map_items::iterator iterChildren = group_by.m_group_secondary_fields.m_map_items.begin(); iterChildren != group_by.m_group_secondary_fields.m_map_items.end(); ++iterChildren)
+          {
+            LayoutItem_Field* pField = dynamic_cast<LayoutItem_Field*>(iterChildren->second);
+            if(pField)
+            {
+              fill_full_field_details(table_name, *pField);
+              fieldsToGet.push_back( sharedptr<LayoutItem_Field>( static_cast<LayoutItem_Field*>(pField->clone() ) ) );
+            }
+          }
+
+          if(!fieldsToGet.empty())
+          {
+            report_build_records(table_name, *nodeSecondaryFields, fieldsToGet, where_clause, Glib::ustring(), true /* one record only */);
+          }
+        }
+
+
+        //Get data and add child rows:
+        GlomUtils::type_vecLayoutFields fieldsToGet;
+        for(LayoutGroup::type_map_items::iterator iterChildren = group_by.m_map_items.begin(); iterChildren != group_by.m_map_items.end(); ++iterChildren)
+        {
+          LayoutItem_Field* pField = dynamic_cast<LayoutItem_Field*>(iterChildren->second);
+          if(pField)
+          {
+            fill_full_field_details(table_name, *pField);
+            fieldsToGet.push_back( sharedptr<LayoutItem_Field>( static_cast<LayoutItem_Field*>(pField->clone() ) ) );
+          }
+          else
+          {
+            LayoutItem_GroupBy* pGroupBy = dynamic_cast<LayoutItem_GroupBy*>(iterChildren->second);
+            if(pGroupBy)
+            {
+              //Recurse, adding a sub-groupby block:
+              report_build_groupby(table_name, *nodeGroupBy, *pGroupBy, where_clause);
+            }
+            else
+            {
+              LayoutItem_Summary* pSummary = dynamic_cast<LayoutItem_Summary*>(iterChildren->second);
+              if(pSummary)
+              {
+                //Recurse, adding a summary block:
+                report_build_summary(table_name, *nodeGroupBy, *pSummary, where_clause);
+              }
+            }
+          }
+        }
+
+        if(!fieldsToGet.empty())
+        {
+          //Rows, with data:
+          Glib::ustring sort_clause;
+          if(group_by.get_field_sort_by())
+            sort_clause = group_by.get_field_sort_by()->get_name(); //TODO: Deal with related fields too.
+
+          report_build_records(table_name, *nodeGroupBy, fieldsToGet, where_clause, sort_clause);
+        }
+      }
+    }
+  }
+}
+
+void Base_DB::report_build_records(const Glib::ustring& table_name, xmlpp::Element& parent_node, const GlomUtils::type_vecLayoutFields& fieldsToGet, const Glib::ustring& where_clause, const Glib::ustring& sort_clause, bool one_record_only)
+{
+  if(!fieldsToGet.empty())
+  {
+    //Field headings:
+    for(GlomUtils::type_vecLayoutFields::const_iterator iter = fieldsToGet.begin(); iter != fieldsToGet.end(); ++iter)
+    {
+      sharedptr<LayoutItem_Field> layout_item = *iter;
+
+      xmlpp::Element* nodeFieldHeading = 0;
+      if(layout_item->m_field.get_glom_type() == Field::TYPE_NUMERIC)
+        nodeFieldHeading = parent_node.add_child("field_heading_numeric"); //This is quite hacky. murrayc.
+      else
+        nodeFieldHeading = parent_node.add_child("field_heading");
+
+      nodeFieldHeading->set_attribute("name", layout_item->get_name()); //Not really necessary, but maybe useful.
+      nodeFieldHeading->set_attribute("title", layout_item->get_title_or_name());
+    }
+
+    Glib::ustring sql_query = GlomUtils::build_sql_select_with_where_clause(table_name,
+      fieldsToGet,
+      where_clause, sort_clause);
+
+    //For instance, when we just want to get a name corresponding to a contact ID, and want to ignore duplicates.
+    if(one_record_only)
+      sql_query += "LIMIT 1";
+
+    bool records_found = false;
+    Glib::RefPtr<Gnome::Gda::DataModel> datamodel = Query_execute(sql_query);
+    if(datamodel)
+    {
+      guint rows_count = datamodel->get_n_rows();
+      if(rows_count > 0)
+        records_found = true;
+
+      for(guint row = 0; row < rows_count; ++row)
+      {
+        xmlpp::Element* nodeRow = parent_node.add_child("row");
+
+        for(guint col = 0; col < fieldsToGet.size(); ++col)
+        {
+          sharedptr<LayoutItem_Field> field = fieldsToGet[col];
+          const Field::glom_field_type field_type = field->m_field.get_glom_type();
+
+          xmlpp::Element* nodeField = 0;
+          if(field_type == Field::TYPE_NUMERIC)
+             nodeField = nodeRow->add_child("field_numeric"); //TODO: I would prefer just to add a field_type attribute to the "field" node and use an <xsl::if> instead. murrayc.
+          else
+             nodeField = nodeRow->add_child("field");
+
+          nodeField->set_attribute("name", field->get_name()); //Not really necessary, but maybe useful.
+
+          Glib::ustring text_value = GlomConversions::get_text_for_gda_value(field_type, datamodel->get_value_at(col, row), field->get_formatting_used().m_numeric_format);
+
+          //The Postgres summary functions return NULL when summarising NULL records, but 0 is more sensible:
+          if(text_value.empty() && dynamic_cast<LayoutItem_FieldSummary*>(field.obj()) && (field_type == Field::TYPE_NUMERIC))
+          {
+            //Use get_text_for_gda_value() instead of "0" so we get the correct numerical formatting:
+            Gnome::Gda::Value value = GlomConversions::parse_value(0);
+            text_value = GlomConversions::get_text_for_gda_value(field_type, value, field->get_formatting_used().m_numeric_format);
+          }
+
+          nodeField->set_attribute("value", text_value);
+        }
+      }
+
+    }
+
+    //If there are no records, show zero
+    //if(!rows_found && show_null_row)
+  }
+}
+
+
+void Base_DB::report_build(const Glib::ustring& table_name, const Report& report, const Glib::ustring& where_clause)
+{
+  //Create a DOM Document with the XML:
+  xmlpp::DomParser dom_parser;;
+
+  xmlpp::Document* pDocument = dom_parser.get_document();
+  xmlpp::Element* nodeRoot = pDocument->get_root_node();
+  if(!nodeRoot)
+  {
+    //Add it if it isn't there already:
+    nodeRoot = pDocument->create_root_node("report_print");
+  }
+
+  Glib::ustring table_title = get_document()->get_table_title(table_name);
+  if(table_title.empty())
+    table_title = table_name;
+
+  nodeRoot->set_attribute("table", table_title);
+
+
+  //The groups:
+  xmlpp::Element* nodeParent = nodeRoot;
+
+
+  nodeRoot->set_attribute("title", report.get_title_or_name());
+
+  GlomUtils::type_vecLayoutFields fieldsToGet_TopLevel;
+
+  for(LayoutGroup::type_map_items::const_iterator iter = report.m_layout_group.m_map_items.begin(); iter != report.m_layout_group.m_map_items.end(); ++iter)
+  {
+    LayoutItem* pPart = iter->second;
+
+    //The Group, and the details for each record in the group:
+    LayoutItem_GroupBy* pGroupBy = dynamic_cast<LayoutItem_GroupBy*>(pPart);
+    if(pGroupBy)
+      report_build_groupby(table_name, *nodeParent, *pGroupBy, where_clause);
+    else
+    {
+      LayoutItem_Summary* pSummary = dynamic_cast<LayoutItem_Summary*>(pPart);
+      if(pSummary)
+      {
+        //Recurse, adding a summary block:
+        report_build_summary(table_name, *nodeParent, *pSummary, where_clause);
+      }
+      else
+      {
+        LayoutItem_Field* pField = dynamic_cast<LayoutItem_Field*>(pPart);
+        if(pField)
+        {
+          fieldsToGet_TopLevel.push_back( sharedptr<LayoutItem_Field>( static_cast<LayoutItem_Field*>(pField->clone() ) ) );
+        }
+      }
+    }
+  }
+
+  //Add top-level records, outside of any groupby or summary, if fields have been specified:
+  if(!fieldsToGet_TopLevel.empty())
+  {
+    xmlpp::Element* nodeGroupBy = nodeParent->add_child("ungrouped_records");
+    report_build_records(table_name, *nodeGroupBy, fieldsToGet_TopLevel, where_clause, Glib::ustring() /* no sort clause */);
+  }
+
+  GlomUtils::transform_and_open(*pDocument, "print_report_to_html.xsl");
+}
