@@ -296,7 +296,7 @@ Gnome::Gda::Value GlomConversions::parse_value(Field::glom_field_type glom_type,
     gda_date.year = the_c_time.tm_year + 1900; //The C time starts at 1900.
     gda_date.month = the_c_time.tm_mon + 1; //The C month starts at 0.
     gda_date.day = the_c_time.tm_mday; //THe C mday starts at 1.
-
+    
     return Gnome::Gda::Value(gda_date);
   }
   else if(glom_type == Field::TYPE_TIME)
@@ -419,10 +419,17 @@ tm GlomConversions::parse_date(const Glib::ustring& text, const std::locale& loc
     date.set_parse(text); //I think this uses the current locale. murrayc.
     if(date.valid())
     {
-      tm the_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
-      the_c_time.tm_year = date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
-      the_c_time.tm_mon = date.get_month() - 1; //C months start at 0.
-      the_c_time.tm_mday = date.get_day(); //starts at 1
+      tm null_c_time = {0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
+      the_c_time = null_c_time;
+      
+      if(date.get_year() != Glib::Date::BAD_YEAR)
+        the_c_time.tm_year = date.get_year() - 1900; //C years start are the AD year - 1900. So, 01 is 1901.
+      
+      if(date.get_month() != Glib::Date::BAD_MONTH)
+        the_c_time.tm_mon = date.get_month() - 1; //C months start at 0.
+        
+      if(date.get_day() != Glib::Date::BAD_DAY)
+        the_c_time.tm_mday = date.get_day(); //starts at 1
 
       success = true;
     }
@@ -600,4 +607,210 @@ Gnome::Gda::Value GlomConversions::get_example_value(Field::glom_field_type fiel
 }
 
 
+unsigned char *
+Glom_PQescapeBytea(const unsigned char *bintext, size_t binlen, size_t *bytealen)
+{
+  const unsigned char *vp;
+  unsigned char *rp;
+  unsigned char *result;
+  size_t    i;
+  size_t    len;
+
+  /*
+   * empty string has 1 char ('\0')
+   */
+  len = 1;
+
+  vp = bintext;
+  for (i = binlen; i > 0; i--, vp++)
+  {
+    if (*vp < 0x20 || *vp > 0x7e)
+      len += 5;     /* '5' is for '\\ooo' */
+    else if (*vp == '\'')
+      len += 2;
+    else if (*vp == '\\')
+      len += 4;
+    else
+      len++;
+  }
+
+  rp = result = (unsigned char *) malloc(len);
+  if (rp == NULL)
+    return NULL;
+
+  vp = bintext;
+  *bytealen = len;
+
+  for (i = binlen; i > 0; i--, vp++)
+  {
+    if (*vp < 0x20 || *vp > 0x7e)
+    {
+      (void) sprintf((char*)rp, "\\\\%03o", *vp);
+      rp += 5;
+    }
+    else if (*vp == '\'')
+    {
+      rp[0] = '\\';
+      rp[1] = '\'';
+      rp += 2;
+    }
+    else if (*vp == '\\')
+    {
+      rp[0] = '\\';
+      rp[1] = '\\';
+      rp[2] = '\\';
+      rp[3] = '\\';
+      rp += 4;
+    }
+    else
+      *rp++ = *vp;
+  }
+  *rp = '\0';
+
+  return result;
+}
+
+#define ISFIRSTOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '3')
+#define ISOCTDIGIT(CH) ((CH) >= '0' && (CH) <= '7')
+#define OCTVAL(CH) ((CH) - '0')
+
+unsigned char *
+Glom_PQunescapeBytea(const unsigned char *strtext, size_t *retbuflen)
+{
+  size_t    strtextlen,
+        buflen;
+  unsigned char *buffer,
+         *tmpbuf;
+  size_t    i,
+        j;
+
+  if (strtext == NULL)
+    return NULL;
+
+  strtextlen = strlen((const char*)strtext);
+
+  /*
+   * Length of input is max length of output, but add one to avoid
+   * unportable malloc(0) if input is zero-length.
+   */
+  buffer = (unsigned char *) malloc(strtextlen + 1);
+  if (buffer == NULL)
+    return NULL;
+
+  for (i = j = 0; i < strtextlen;)
+  {
+    switch (strtext[i])
+    {
+      case '\\':
+        i++;
+        if (strtext[i] == '\\')
+          buffer[j++] = strtext[i++];
+        else
+        {
+          if ((ISFIRSTOCTDIGIT(strtext[i])) &&
+            (ISOCTDIGIT(strtext[i + 1])) &&
+            (ISOCTDIGIT(strtext[i + 2])))
+          {
+            int     byte;
+
+            byte = OCTVAL(strtext[i++]);
+            byte = (byte << 3) + OCTVAL(strtext[i++]);
+            byte = (byte << 3) + OCTVAL(strtext[i++]);
+            buffer[j++] = byte;
+          }
+        }
+
+        /*
+         * Note: if we see '\' followed by something that isn't a
+         * recognized escape sequence, we loop around having done
+         * nothing except advance i.  Therefore the something will
+         * be emitted as ordinary data on the next cycle. Corner
+         * case: '\' at end of string will just be discarded.
+         */
+        break;
+
+      default:
+        buffer[j++] = strtext[i++];
+        break;
+    }
+  }
+  buflen = j;         /* buflen is the length of the dequoted
+                 * data */
+
+  /* Shrink the buffer to be no larger than necessary */
+  /* +1 avoids unportable behavior when buflen==0 */
+  tmpbuf = (unsigned char*)realloc(buffer, buflen + 1);
+
+  /* It would only be a very brain-dead realloc that could fail, but... */
+  if (!tmpbuf)
+  {
+    free(buffer);
+    return NULL;
+  }
+
+  *retbuflen = buflen;
+  return tmpbuf;
+}
+
+Glib::ustring GlomConversions::get_escaped_binary_data(guint8* buffer, size_t buffer_size)
+{
+  //g_warning("GlomConversions::get_escaped_binary_data: debug: buffer ");
+  //for(int i = 0; i < 10; ++i)
+  //  g_warning("%02X (%c), ", (guint8)buffer[i], buffer[i]);
+        
+  //TODO: Performance: Preallocate a string of the appropriate size.
+  //Use an output parameter instead of copying it during return.
+  
+  Glib::ustring result;
+  
+  /* This should be much faster, and more correct, but it gives an error - "unterminated quoted string at or near: "
+  size_t escaped_length = 0;
+  guchar* escaped =  Glom_PQescapeBytea(buffer, buffer_size, &escaped_length);
+  
+  if(escaped && escaped_length)
+  {
+    result = std::string((char*)escaped, escaped_length);
+    free(escaped);
+  }
+  */
+  
+  
+  if(buffer && buffer_size)
+  {
+    guint8* buffer_end = buffer + buffer_size;
+    char byte_as_octal[4]; //3 digits, and a null terminator
+    
+    for(guint8* pos = buffer; pos < buffer_end; ++pos)
+    {
+      sprintf(byte_as_octal, "%03o", *pos); //Format as octal with 3 digits.
+      byte_as_octal[3] = 0;
+      
+//      g_warning("byte=%d, as_hex=%s", *pos, byte_as_octal);      
+
+      result += Glib::ustring("\\\\") + byte_as_octal;
+    }
+  }
+ 
+  return result;
+}
+
+Gnome::Gda::Value GlomConversions::parse_escaped_binary_data(const Glib::ustring& escaped_data)
+{
+  //Hopefully we don't need to use this because Gda does it for us when we read a part of a "SELECT" result into a Gnome::Value.
+  //TODO: Performance
+  
+  Gnome::Gda::Value result;
+  size_t buffer_binary_length = 0;
+  guchar* buffer_binary =  Glom_PQunescapeBytea((guchar*)escaped_data.c_str(), &buffer_binary_length);
+  if(buffer_binary)
+  {
+    result.set(buffer_binary, buffer_binary_length);
+    free(buffer_binary);
+  }
+  
+  return result;
+}
+
+
+
 
