@@ -108,6 +108,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   pack_start(m_HBox, Gtk::PACK_SHRINK);
 
   m_FlowTable.signal_field_edited().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_edited) );
+  m_FlowTable.signal_field_open_details_requested().connect( sigc::mem_fun(*this,  &Box_Data_Details::on_flowtable_field_open_details_requested) );
   show_all();
 
   m_FlowTable.signal_related_record_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_related_record_changed) );
@@ -115,6 +116,8 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 
   m_FlowTable.signal_layout_changed().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_layout_changed) );
 
+  m_FlowTable.signal_requested_related_details().connect( sigc::mem_fun(*this, &Box_Data_Details::on_flowtable_requested_related_details) );
+    
   m_ignore_signals = false;
 }
 
@@ -163,7 +166,7 @@ void Box_Data_Details::create_layout()
     m_FlowTable.set_table(m_strTableName); //This allows portals to get full Relationship information
 
     //This map of layout groups will also contain the field information from the database:
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
 
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
@@ -344,25 +347,9 @@ void Box_Data_Details::set_entered_field_data(const LayoutItem_Field& field, con
 
 Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
 {
-
-  Glib::ustring strResult;
-/* TODO_port.
-  guint uiRow = 0;
-  bool bPresent = get_field_primary_key(uiRow);
-  if(bPresent)
-    strResult = m_AddDel.get_value(uiRow);
-
-*/
-  return Gnome::Gda::Value(strResult);
+  return m_primary_key_value;
 }
 
-/*
-void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_value, Glib::ustring strTableName)
-{
-  signal_user_requested_related_details().emit(strTableName, key_value);
-}
-*/
-
 void Box_Data_Details::recalculate_fields_for_related_records(const Glib::ustring& relationship_name)
 {
   m_FieldsCalculationInProgress.clear();
@@ -448,23 +435,22 @@ Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_del
   return m_signal_record_deleted; 
 }
 
-/*
-Box_Data_Details::type_signal_user_requested_related_details Box_Data_Details::signal_user_requested_related_details()
+Box_Data_Details::type_signal_requested_related_details Box_Data_Details::signal_requested_related_details()
 {
-  return m_signal_user_requested_related_details;
+  return m_signal_requested_related_details;
 }
-*/
 
 void Box_Data_Details::on_flowtable_layout_changed()
 {
   //Get new layout:
   Document_Glom::type_mapLayoutGroupSequence layout_groups;
   m_FlowTable.get_layout_groups(layout_groups);
-
+  
+  //Store it in the document:
   Document_Glom* document = get_document();
   if(document)
-    document->set_data_layout_groups("details", m_strTableName, layout_groups);
-
+    document->set_data_layout_groups(m_layout_name, m_strTableName, layout_groups);
+  
   //Build the view again from the new layout:
   create_layout();
 
@@ -472,11 +458,32 @@ void Box_Data_Details::on_flowtable_layout_changed()
   fill_from_database();
 }
 
+void Box_Data_Details::on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
+{
+  if(GlomConversions::value_is_empty(primary_key_value))
+    return; //Ignore empty ID fields.
+    
+  signal_requested_related_details().emit(table_name, primary_key_value);
+}
+
 void Box_Data_Details::on_flowtable_related_record_changed(const Glib::ustring& relationship_name)
 {
   recalculate_fields_for_related_records(relationship_name);
 }
 
+void Box_Data_Details::on_flowtable_field_open_details_requested(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
+{
+  if(GlomConversions::value_is_empty(field_value))
+    return; //Ignore empty ID fields.
+    
+  Relationship relationship;
+  const bool related_to_one = get_document()->get_field_used_in_relationship_to_one(m_strTableName, layout_field.get_name(), relationship);
+  if(related_to_one)
+  {
+    signal_requested_related_details().emit(relationship.get_to_table(), field_value);
+  }
+}
+
 void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_field, const Gnome::Gda::Value& field_value)
 {
   if(m_ignore_signals)
@@ -540,13 +547,14 @@ void Box_Data_Details::on_flowtable_field_edited(const LayoutItem_Field& layout_
     //const Field& field = layout_field.m_field;
 
     //Set the value in all instances of this field in the layout (The field might be on the layout more than once):
-    if(layout_field.m_field.get_glom_type() != Field::TYPE_IMAGE) //TODOL For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
+    if(layout_field.m_field.get_glom_type() != Field::TYPE_IMAGE) //TODO For now, don't do this for images, because the ImageGlom widget expects a broken GdaValue, because gda_value_get_binary() needs to be fixed.
       m_FlowTable.set_field_value(layout_field, field_value);
 
     //Update the field in the record (the record with this primary key):
     try
     {
       bool bTest = set_field_value_in_database(layout_field, field_value, primary_key_field, primary_key_value);
+
       //Glib::ustring strQuery = "UPDATE " + table_name;
       //strQuery += " SET " +  /* table_name + "." + postgres does not seem to like the table name here */ strFieldName + " = " + field.sql(field_value);
       //strQuery += " WHERE " + table_name + "." + primary_key_field.get_name() + " = " + primary_key_field.sql(primary_key_value);
@@ -639,6 +647,8 @@ void Box_Data_Details::refresh_related_fields(const Gtk::TreeModel::iterator& /*
   {
     const Glib::ustring query = build_sql_select(m_strTableName, fieldsToGet, primary_key, primary_key_value);
 
+    //g_warning("Box_Data_Details::refresh_related_fields(): get query = %s", query.c_str());
+    
     Glib::RefPtr<Gnome::Gda::DataModel> result = Query_execute(query);
     if(!result)
       handle_error();
@@ -708,7 +718,7 @@ void Box_Data_Details::on_userlevel_changed(AppState::userlevels user_level)
 bool Box_Data_Details::get_field_primary_key(Field& field) const
 {
   field = m_field_primary_key;
-  return false;
+  return true;
 }
 
 void Box_Data_Details::print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group)
@@ -797,7 +807,7 @@ void Box_Data_Details::print_layout()
     //The groups:
     xmlpp::Element* nodeParent = nodeRoot;
 
-    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
+    Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups(m_layout_name);
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
     {
       const LayoutGroup& layout_group = iter->second;
