@@ -72,6 +72,16 @@ Box_Data_List::~Box_Data_List()
   remove_view(&m_AddDel);
 }
 
+void Box_Data_List::enable_buttons()
+{
+  const Privileges table_privs = get_current_privs(m_strTableName);
+  
+    //Enable/Disable record creation and deletion:
+  m_AddDel.set_allow_add(table_privs.m_create);
+  m_AddDel.set_allow_delete(table_privs.m_delete);
+  m_AddDel.set_allow_view_details(table_privs.m_view);
+}
+
 bool Box_Data_List::fill_from_database()
 {
   bool result = false;
@@ -96,9 +106,7 @@ bool Box_Data_List::fill_from_database()
     //Do not try to show the data if the user may not view it:
     const Privileges table_privs = get_current_privs(m_strTableName);
 
-    //Enable/Disable record creation and deletion:
-    m_AddDel.set_allow_add(table_privs.m_create);
-    m_AddDel.set_allow_delete(table_privs.m_delete);
+    enable_buttons();
 
     m_AddDel.set_where_clause(m_strWhereClause);
 
@@ -128,6 +136,10 @@ bool Box_Data_List::fill_from_database()
 
 void Box_Data_List::on_adddel_user_requested_add()
 {
+g_warning("Box_Data_List::on_adddel_user_requested_add() start: m_FieldsShowns.size()=%d", m_FieldsShown.size());
+  if(m_FieldsShown.empty())
+    return; //Don't try to add a record to a list with no fields.
+    
   Gtk::TreeModel::iterator iter = m_AddDel.get_item_placeholder();
   if(iter)
   {
@@ -168,6 +180,8 @@ void Box_Data_List::on_adddel_user_requested_add()
       //g_warning("Box_Data_List::on_adddel_user_requested_add(): index_field_to_edit does not exist: %d", index_field_to_edit);
     }
   }
+  
+  g_warning("Box_Data_List::on_adddel_user_requested_add() end");
 }
 
 void Box_Data_List::on_adddel_user_requested_edit(const Gtk::TreeModel::iterator& row)
@@ -438,8 +452,6 @@ void Box_Data_List::refresh_related_fields(const Gtk::TreeModel::iterator& row,
       //Field contents:
       if(result->get_n_rows())
       {
-        g_warning("Box_Data_List::refresh_related_fields(): records_count=%d", result->get_n_rows());
-
         type_vecLayoutFields::const_iterator iterFields = fieldsToGet.begin();
 
         guint cols_count = result->get_n_columns();
@@ -633,7 +645,6 @@ void Box_Data_List::create_layout()
       m_AddDel.set_key_field(field_primary_key);
 
       m_FieldsShown = get_fields_to_show();
-      //g_warning("m_Fields[0].m_field.get_name() = %s", m_FieldsShown[0].m_field.get_name().c_str());
 
       //Add extra possibly-non-visible columns that we need:
       //TODO: Only add it if it is not already there.
