@@ -51,7 +51,8 @@ public:
 
 protected:
   virtual void create_layout(); //override
-  virtual void fill_from_database(); //override.
+  virtual bool fill_from_database(); //override.
+  virtual void enable_buttons();
 
   virtual bool get_field_primary_key_index(guint& field_column) const; //TODO: visible 
   virtual bool get_field_primary_key(Field& field) const;
@@ -75,6 +76,7 @@ protected:
 
   virtual bool get_field_column_index(const Glib::ustring& field_name, guint& index) const;
 
+  virtual void print_layout();
   virtual void print_layout_group(xmlpp::Element* node_parent, const LayoutGroup& group);
 
   //Member widgers:
