@@ -150,19 +150,18 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
     {
       Gtk::TreeModel::Row rowChild = *iterChild;
 
-      if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
+      if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_PORTAL)
+      {
+        const LayoutItem_Portal portal = rowChild[m_model_items->m_columns.m_col_portal];
+        group.add_item(portal);
+      }
+      else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
       {
         //Recurse:
         LayoutGroup group_child;
         fill_group(iterChild, group_child);
         group.add_item(group_child);
       }
-      else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_PORTAL)
-      {
-        LayoutItem_Portal portal;
-        portal.set_relationship(  rowChild[m_model_items->m_columns.m_col_relationship] );
-        group.add_item(portal);
-      }
       else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD)
       {
         //Add field:
@@ -170,7 +169,7 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
 
         field.set_name( rowChild[m_model_items->m_columns.m_col_name] );
 
-        field.m_relationship = rowChild[m_model_items->m_columns.m_col_relationship_name];
+        field.m_relationship = rowChild[m_model_items->m_columns.m_col_relationship];
 
         //if(!relationship_name.empty())
         //{
@@ -214,19 +213,20 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
     for(LayoutGroup::type_map_const_items::const_iterator iter = items.begin(); iter != items.end(); ++iter)
     {
       const LayoutItem* item = iter->second;
-      const LayoutGroup* child_group = dynamic_cast<const LayoutGroup*>(item);
-      if(child_group) //If it is a group:
-        add_group(iterNewGroup, *child_group); //recursive
+  
+      const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
+      if(portal) //If it is a portal
+      {
+        Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
+        Gtk::TreeModel::Row row = *iterField;
+        row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
+        row[m_model_items->m_columns.m_col_portal] = *portal;
+      }
       else
       {
-        const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
-        if(portal) //If it is a portal
-        {
-          Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
-          Gtk::TreeModel::Row row = *iterField;
-          row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
-          row[m_model_items->m_columns.m_col_relationship] = portal->get_relationship();
-        }
+        const LayoutGroup* child_group = dynamic_cast<const LayoutGroup*>(item);
+        if(child_group) //If it is a group:
+          add_group(iterNewGroup, *child_group); //recursive
         else
         {
           const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
@@ -238,8 +238,8 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
             row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
             row[m_model_items->m_columns.m_col_field_formatting] = *field;
             row[m_model_items->m_columns.m_col_name] = field->get_name();
-            row[m_model_items->m_columns.m_col_relationship_name] = field->m_relationship;
-
+            row[m_model_items->m_columns.m_col_relationship] = field->m_relationship;
+  
             row[m_model_items->m_columns.m_col_editable] = field->get_editable();
           }
         }
@@ -464,7 +464,7 @@ void Dialog_Layout_Details::on_button_field_add()
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
       row[m_model_items->m_columns.m_col_name] = layout_item.get_name();
-      row[m_model_items->m_columns.m_col_relationship_name] = layout_item.m_relationship;
+      row[m_model_items->m_columns.m_col_relationship] = layout_item.m_relationship;
       row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
@@ -611,7 +611,10 @@ void Dialog_Layout_Details::on_button_add_related()
     {
       Gtk::TreeModel::Row row = *iter;
       row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
-      row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+      
+      LayoutItem_Portal portal;
+      portal.set_relationship(relationship.get_name());
+      row[m_model_items->m_columns.m_col_portal] = portal;
       row[m_model_items->m_columns.m_col_editable] = true; //A sane default.
       //row[m_model_items->m_columns.m_col_title] = field.get_title();
 
@@ -766,7 +769,7 @@ void Dialog_Layout_Details::on_button_field_edit()
         {
           LayoutItem_Field field;
           field.set_name( row[m_model_items->m_columns.m_col_name] ); //Start with this one selected.
-          field.m_relationship = row[m_model_items->m_columns.m_col_relationship_name];
+          field.m_relationship = row[m_model_items->m_columns.m_col_relationship];
 
           //if(!relationship_name.empty())
           //{
@@ -778,7 +781,7 @@ void Dialog_Layout_Details::on_button_field_edit()
           if(test)
           {
             row[m_model_items->m_columns.m_col_name] = field.get_name();
-            row[m_model_items->m_columns.m_col_relationship_name] = field.m_relationship;
+            row[m_model_items->m_columns.m_col_relationship] = field.m_relationship;
 
             row[m_model_items->m_columns.m_col_editable] = field.get_editable();
           }
@@ -788,11 +791,13 @@ void Dialog_Layout_Details::on_button_field_edit()
         case TreeStore_Layout::TYPE_PORTAL:
         {
           Relationship relationship;
-          relationship.set_name( row[m_model_items->m_columns.m_col_relationship] ); //Start with this one selected.
+          LayoutItem_Portal portal = row[m_model_items->m_columns.m_col_portal];
+          relationship.set_name( portal.get_relationship() ); //Start with this one selected.
           bool test = offer_relationship_list(relationship);
           if(test)
           {
-            row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+            portal.set_relationship(relationship.get_name());
+            row[m_model_items->m_columns.m_col_portal] = portal;
           }
 
           break;
@@ -876,14 +881,15 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
       }
       else if(is_relationship)
       {
-        markup = _("Related: ") + row[m_model_items->m_columns.m_col_relationship];
+        const LayoutItem_Portal portal = row[m_model_items->m_columns.m_col_portal];
+        markup = _("Related: ") + portal.get_relationship();
       }
       else
       {
         //It's a field:
 
         //Indicate that it's a field in another table.
-        const Relationship& rel = row[m_model_items->m_columns.m_col_relationship_name];
+        const Relationship& rel = row[m_model_items->m_columns.m_col_relationship];
         const Glib::ustring relationship = rel.get_name();
         if(!relationship.empty())
           markup = relationship + "::";
@@ -897,7 +903,10 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
 
       renderer_text->property_markup() = markup;
 
-      renderer_text->property_editable() = false; //Names can never be edited.
+      if(is_group)
+        renderer_text->property_editable() = true; //Group names can be changed.
+      else
+        renderer_text->property_editable() = false; //Field names can never be edited.
     }
   }
 }
