@@ -19,26 +19,22 @@
  */
 
 #include "notebook_data.h"
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Notebook_Data::Notebook_Data()
 {
   //Add Pages:
-  pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_List, gettext("List")));
+  pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_List, _("List")));
   m_iPage_List = 0;
 
-  pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_Details, gettext("Details")));
+  pages().push_back(Gtk::Notebook_Helpers::TabElem(m_Box_Details, _("Details")));
   m_iPage_Details = 1;
 
 
   //Connect signals:
 
   //Allow List to ask Details to show a record.
-  m_Box_List.signal_user_requested_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_Details_user_requested_details));
-
-  //Allow Details's Related Records to ask Details to show a record.
-  //m_Box_Details.signal_user_requested_related_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_Details_user_requested_related_details));
-
+  m_Box_List.signal_user_requested_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_list_user_requested_details));
     
   //Allow Details to ask List to ask Details to show a different record:
   m_Box_Details.signal_nav_first().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_details_nav_first));
@@ -49,6 +45,9 @@ Notebook_Data::Notebook_Data()
   //Allow Details to tell List about record deletion:
   m_Box_Details.signal_record_deleted().connect(sigc::mem_fun(m_Box_List, &Box_Data_List::on_Details_record_deleted));
 
+  //Allow Details to ask to show a different record in a different table:
+  m_Box_Details.signal_requested_related_details().connect(sigc::mem_fun(*this, &Notebook_Data::on_Details_user_requested_related_details));
+
   //Fill composite view:
   add_view(&m_Box_List);
   add_view(&m_Box_Details);
@@ -58,38 +57,47 @@ Notebook_Data::Notebook_Data()
 
 Notebook_Data::~Notebook_Data()
 {
+  remove_view(&m_Box_List);
+  remove_view(&m_Box_Details);
 }
 
-void Notebook_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
+bool Notebook_Data::init_db_details(const Glib::ustring& strTableName, const Glib::ustring& strWhereClause)
 {
+  bool result = false;
   //strWhereClause is only used as a result of a find.
 
   //Performance optimisation:
   //Keep the connection open during all these operations:
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
-  
-    m_Box_List.init_db_details(strTableName, strWhereClause);
+
+    result = m_Box_List.init_db_details(strTableName, strWhereClause);
     m_Box_Details.init_db_details(strTableName, m_Box_List.get_primary_key_value_selected());
   }
 
   //Select List as default:
   set_current_page(m_iPage_List);
+
+  return result;
 }
 
-void Notebook_Data::on_Details_user_requested_details(Gnome::Gda::Value primary_key_value)
+void Notebook_Data::on_list_user_requested_details(Gnome::Gda::Value primary_key_value)
 {
-  m_Box_Details.refresh_db_details(primary_key_value);
+  m_Box_Details.refresh_data_from_database(primary_key_value);
   set_current_page(m_iPage_Details);
 }
 
-void Notebook_Data::on_Details_user_requested_related_details(Glib::ustring strTableName, Gnome::Gda::Value primary_key_value)
+void Notebook_Data::on_Details_user_requested_related_details(const Glib::ustring& strTableName, Gnome::Gda::Value primary_key_value)
 {
+  signal_record_details_requested().emit(strTableName, primary_key_value);
+  
+  /*
   //Show a different table:
   init_db_details(strTableName);
 
   //Show the specific record:
-  on_Details_user_requested_details(primary_key_value);
+  on_list_user_requested_details(primary_key_value);
+  */
 }
 
 void Notebook_Data::select_page_for_find_results()
@@ -117,6 +125,19 @@ void Notebook_Data::do_menu_developer_layout()
   } 
 }
 
+void Notebook_Data::do_menu_file_print()
+{
+  int iPageCurrent = get_current_page();
+
+  Gtk::Widget* pChild  = get_nth_page(iPageCurrent);
+  if(pChild)
+  {
+    Box_Data* pBox = dynamic_cast<Box_Data*>(pChild);
+    if(pBox)
+      pBox->print_layout();
+  } 
+}
+
 enum dataview
 {
   DATA_VIEW_Details,
@@ -134,3 +155,13 @@ Notebook_Data::dataview Notebook_Data::get_current_view() const
   return result;
 }
 
+Notebook_Data::type_signal_record_details_requested Notebook_Data::signal_record_details_requested()
+{
+  return m_signal_record_details_requested;
+}
+
+void Notebook_Data::show_details(const Gnome::Gda::Value& primary_key_value)
+{
+  //Reuse this implementation:
+  on_list_user_requested_details(primary_key_value);
+}
