@@ -79,51 +79,53 @@ void FlowTableWithFields::add_layout_item_at_position(const LayoutItem& item, co
   const LayoutItem* pItem = &item;
 
   //Get derived type and do the appropriate thing:
-  const LayoutGroup* group = dynamic_cast<const LayoutGroup*>(pItem);
-  if(group)
-    add_layout_group_at_position(*group, add_before);
+  const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(pItem);
+  if(field)
+  {
+    add_field_at_position(*field, m_table_name, add_before);
+
+    //Do not allow editing of auto-increment fields:
+    if(field->m_field.get_auto_increment())
+      set_field_editable(*field, false);
+    else
+      set_field_editable(*field, field->get_editable_and_allowed());
+  }
   else
   {
-    const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(pItem);
-    if(field)
+    const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(pItem);
+    if(portal)
     {
-      add_field_at_position(*field, m_table_name, add_before);
+      Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
+      if(pDocument)
+      {
+        Relationship relationship;
+        bool found = pDocument->get_relationship(m_table_name, portal->get_relationship(), relationship);
+        if(found)
+        {
+          Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
+          add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
 
-      //Do not allow editing of auto-increment fields:
-      if(field->m_field.get_auto_increment())
-        set_field_editable(*field, false);
-      else
-        set_field_editable(*field, field->get_editable_and_allowed());
+          portal_box->init_db_details(*portal); //Create the layout
+
+          portal_box->set_layout_item(portal->clone(), relationship.get_to_table());
+          portal_box->show();
+          add(*portal_box, true /* expand */);
+
+          m_portals.push_back(portal_box);
+          add_layoutwidgetbase(portal_box, add_before);
+
+          //Connect signals:
+          portal_box->signal_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_record_changed) );
+          
+          portal_box->signal_user_requested_details().connect( sigc::bind( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_user_requested_details), portal_box));
+        }
+      }
     }
     else
     {
-      const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(pItem);
-      if(portal)
-      {
-        Document_Glom* pDocument = static_cast<Document_Glom*>(get_document());
-        if(pDocument)
-        {
-          Relationship relationship;
-          bool found = pDocument->get_relationship(m_table_name, portal->get_relationship(), relationship);
-          if(found)
-          {
-            Box_Data_List_Related* portal_box = Gtk::manage(new Box_Data_List_Related);
-            add_view(portal_box); //Give it access to the document, needed to get the layout and fields information.
-
-            portal_box->init_db_details(relationship); //Create the layout
-
-            portal_box->set_layout_item(portal->clone(), relationship.get_to_table());
-            portal_box->show();
-            add(*portal_box, true /* expand */);
-
-            m_portals.push_back(portal_box);
-            add_layoutwidgetbase(portal_box, add_before);
-
-            //Connect signals:
-            portal_box->signal_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_portal_record_changed) );
-          }
-        }
-      }
+     const LayoutGroup* group = dynamic_cast<const LayoutGroup*>(pItem);
+      if(group)
+        add_layout_group_at_position(*group, add_before);
     }
   }
 }
@@ -186,7 +188,9 @@ void FlowTableWithFields::add_layout_group_at_position(const LayoutGroup& group,
 
     //Connect signal:
     flow_table->signal_field_edited().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_edited) );
+    flow_table->signal_field_open_details_requested().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_entry_open_details_requested) );
     flow_table->signal_related_record_changed().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_related_record_changed) );
+    flow_table->signal_requested_related_details().connect( sigc::mem_fun(*this, &FlowTableWithFields::on_flowtable_requested_related_details) );
   }
 }
 
@@ -277,11 +281,18 @@ void FlowTableWithFields::add_field_at_position(const LayoutItem_Field& layoutit
 
   //info.m_group = layoutitem_field.m_group;
 
-  add(*(info.m_first), *(info.m_second));
+  //Expand multiline text fields to take up the maximum possible width:
+  bool expand_second = false;
+  if( (layoutitem_field.m_field.get_glom_type() == Field::TYPE_TEXT) && layoutitem_field.get_formatting_used().get_text_format_multiline())
+    expand_second = true;
+
+  add(*(info.m_first), *(info.m_second), expand_second);
 
   info.m_second->signal_edited().connect( sigc::bind(sigc::mem_fun(*this, &FlowTableWithFields::on_entry_edited), layoutitem_field)  ); //TODO:  Is it a good idea to bind the LayoutItem? sigc::bind() probably stores a copy at this point.
   info.m_second->signal_layout_item_added().connect( sigc::bind(
     sigc::mem_fun(*this, &FlowTableWithFields::on_datawidget_layout_item_added), info.m_second) );
+    
+  info.m_second->signal_open_details_requested().connect( sigc::bind(sigc::mem_fun(*this, &FlowTableWithFields::on_entry_open_details_requested), layoutitem_field)  );
 
   m_listFields.push_back(info); //This would be the wrong position, but you should only use this method directly when you expect it to be followed by a complete re-layout.
 }
@@ -531,22 +542,41 @@ FlowTableWithFields::type_signal_field_edited FlowTableWithFields::signal_field_
   return m_signal_field_edited;
 }
 
+FlowTableWithFields::type_signal_field_open_details_requested FlowTableWithFields::signal_field_open_details_requested()
+{
+  return m_signal_field_open_details_requested;
+}
+
 FlowTableWithFields::type_signal_related_record_changed FlowTableWithFields::signal_related_record_changed()
 {
   return m_signal_related_record_changed;
 }
 
+FlowTableWithFields::type_signal_requested_related_details FlowTableWithFields::signal_requested_related_details()
+{
+  return m_signal_requested_related_details;
+}
 
 void FlowTableWithFields::on_entry_edited(const Gnome::Gda::Value& value, LayoutItem_Field field)
 {
   m_signal_field_edited.emit(field, value);
 }
 
+void FlowTableWithFields::on_entry_open_details_requested(const Gnome::Gda::Value& value, LayoutItem_Field field)
+{
+  m_signal_field_open_details_requested.emit(field, value);
+}
+
 void FlowTableWithFields::on_flowtable_entry_edited(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
 {
   m_signal_field_edited.emit(field, value);
 }
 
+void FlowTableWithFields::on_flowtable_entry_open_details_requested(const LayoutItem_Field& field, const Gnome::Gda::Value& value)
+{
+  m_signal_field_open_details_requested.emit(field, value);
+}
+
 void FlowTableWithFields::set_design_mode(bool value)
 {
   FlowTable::set_design_mode(value);
@@ -578,10 +608,12 @@ void FlowTableWithFields::get_layout_groups(Document_Glom::type_mapLayoutGroupSe
         LayoutGroup group_child;
         pFlowTable->get_layout_group(group_child);
         groups[sequence] = group_child;
+        
         ++sequence;
       }
     }
   }
+ 
 }
 
 void FlowTableWithFields::get_layout_group(LayoutGroup& group)
@@ -615,7 +647,10 @@ void FlowTableWithFields::get_layout_group(LayoutGroup& group)
         const LayoutItem* pLayoutItem = (*iter)->get_layout_item();
 
         if(pLayoutItem)
+        {
           group.add_item(*pLayoutItem);
+        }
+          
       }
     }
   }
@@ -636,6 +671,13 @@ void FlowTableWithFields::add_layoutwidgetbase(LayoutWidgetBase* layout_widget,
 
 void FlowTableWithFields::on_layoutwidget_changed()
 {
+  LayoutGroup* pLayoutItem = dynamic_cast<LayoutGroup*>(get_layout_item());
+  if(pLayoutItem)
+  {
+    //Update tha LayoutItem by looking at the child widgets:
+    get_layout_group(*pLayoutItem);
+  }
+
   //Forward the signal to the container:
   signal_layout_changed().emit();
 }
@@ -720,4 +762,13 @@ void FlowTableWithFields::on_flowtable_related_record_changed(const Glib::ustrin
   signal_related_record_changed().emit(relationship_name);
 }
 
+void FlowTableWithFields::on_portal_user_requested_details(Gnome::Gda::Value primary_key_value, Box_Data_List_Related* portal_box)
+{
+  signal_requested_related_details().emit(portal_box->get_relationship().get_to_table(), primary_key_value);
+}
 
+void FlowTableWithFields::on_flowtable_requested_related_details(const Glib::ustring& table_name, Gnome::Gda::Value primary_key_value)
+{
+  //Forward it to the parent:
+  signal_requested_related_details().emit(table_name, primary_key_value);
+}
